import os
import logging
import shutil
from pathlib import Path
import uuid
import subprocess

from tinytag import TinyTag
from pydantic import BaseModel

from jgutils.filesystem import get_files_recursive

logger = logging.getLogger(__name__)


class Song(BaseModel):

    title: str
    identifier: str
    formats: list[Path] = []

class Album(BaseModel):

    title: str
    songs: list[Song] = []

class Artist(BaseModel):

    name: str
    albums: list[Album] = []
    songs: list[Song] = []

    def get_album(self, title: str) -> Album | None:
        for album in self.albums:
            if album.title == title:
                return album
        return None

class MusicCollection(BaseModel):

    artists: list[Artist] = []

    def get_artist(self, name:str) -> Artist | None:
        for artist in self.artists:
            if artist.name == name:
                return artist
        return None


def ffmpeg_convert(input_path: Path, output_path: Path) -> int:

    identifier = str(uuid.uuid4())
    input_tmp = input_path.parent / (identifier + input_path.suffix)
    shutil.copy(input_path, input_tmp)
    
    output_tmp = output_path.parent / (identifier + output_path.suffix)
    cmd = f"ffmpeg -i '{input_tmp}' -ab 320k -map_metadata 0 -id3v2_version 3 '{output_tmp}'"

    os.makedirs(output_path.parent, exist_ok=True)
    status = subprocess.run(cmd, shell=True)

    input_tmp.unlink()
    if status.returncode == 0:
        shutil.move(output_tmp, output_path)
    else:
        logger.error("Error converting: %s", input_path)
    return status.returncode


def get_metadata(input_dir: Path, extension: str) -> MusicCollection:

    files = get_files_recursive(input_dir, extension)
 
    collection = MusicCollection()

    for eachFile in files:
        tag = TinyTag.get(eachFile)
        if not tag.title:
            logger.warn("Found tag with no title, skipping: %s", tag)
            continue

        artist = collection.get_artist(tag.artist)
        if not artist:
            artist = Artist(name=tag.artist)
            collection.artists.append(artist)

        if tag.album:
            album = artist.get_album(tag.album)
            if not album:
                album = Album(title=tag.album)
                artist.albums.append(album)

        song = Song(title=tag.title, identifier=str(uuid.uuid4()))
        song.formats.append(eachFile.relative_to(input_dir))
        if tag.album:
            album.songs.append(song)
        else:
            artist.songs.append(song)
        
    return collection

def refresh(input_dir: Path):

    files = get_files_recursive(input_dir, "flac")

    for eachFile in files:
        tag = TinyTag.get(eachFile)
        if not tag.title:
            logger.warn("Found tag with no title, skipping: %s", tag)
            continue

        os.makedirs(input_dir / tag.artist, exist_ok=True)

        if tag.album:
            os.makedirs(input_dir / tag.artist / tag.album, exist_ok=True)
            shutil.move(eachFile, input_dir / tag.artist / tag.album / f"{tag.title}.flac")
        else:
            shutil.move(eachFile, input_dir / tag.artist / f"{tag.title}.flac")

       
